// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 
// POV-Ray SDL file for making functions for natural cubic splines
// Copyright 2014 by Tor Olav Kristensen
// Email: t o r . o l a v . k [_A_T_] g m a i l . c o m
// http://subcube.com
// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 

#include "colors.inc"

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 

#declare Times =
  array[5] {
     -2,
     -1,
      0,
      0.5,
      1
  }

#declare Points =
  array[5] {
    <-2, 0, 40>,
    <0, 0, -2>,
    <1, 0, 2>,
    <0, 6, 2>,
    <8, 2, 2>
  }

#declare Start = Times[0];
#declare End = Times[4];
#declare Span = End - Start;

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 
// The built in way
 
#declare NaturalSpline =
  spline {
    natural_spline
    Times[0], Points[0]
    Times[1], Points[1]
    Times[2], Points[2]
    Times[3], Points[3]
    Times[4], Points[4]
  }

#declare Intervals = 200;

#declare I = 0;
#while (I < Intervals)
  #declare T = Start + I/Intervals*Span;
  sphere {
    NaturalSpline(T), 0.05
    pigment { color Red }
  }
  #declare I = I + 1;
#end // while

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 
// Macros for another way

#macro Size1A(Array1A)

  dimension_size(Array1A, 1)

#end // macro Size1A


#macro ExtractComponent1A(Vectors1A, v0)

  #local SizeU = Size1A(Vectors1A);
  #local Component1A = array[SizeU];

  #local I = 0;
  #while (I < SizeU)
    #local Component1A[I] = vdot(Vectors1A[I], v0);
    #local I = I + 1;
  #end // while

  Component1A

#end // macro ExtractComponent1A


#macro NaturalCubicSpline_Function1A(tt, yy)

  #local Size = Size1A(tt);
  #local N = Size - 1;

  #local hh = array[Size];
  #local kk = array[Size][3];

  #local I = 0;
  #while (I <= N - 1)
    #local hh[I] = tt[I+1] - tt[I];
    #local kk[I][1] = 6*(yy[I+1] - yy[I])/hh[I];
    #local I = I + 1;
  #end // while

  #local uu = array[Size];
  #local vv = array[Size];

  #local uu[1] = 2*(hh[0] + hh[1]);
  #local vv[1] = kk[1][1] - kk[0][1];
  #local I = 2;
  #while (I <= N - 1)
    #local uu[I] = 2*(hh[I] + hh[I-1]) - pow(hh[I-1], 2)/uu[I-1];
    #local vv[I] = kk[I][1] - kk[I-1][1] - hh[I-1]*vv[I-1]/uu[I-1];
    #local I = I + 1;
  #end // while

  #local zz = array[Size];

  #local zz[N] = 0;
  #local I = N - 1;
  #while (I >= 1)
    #local zz[I] = (vv[I] - hh[I]*zz[I+1])/uu[I];
    #local I = I - 1;
  #end // while
  #local zz[0] = 0;

  #undef uu
  #undef vv

  #local I = 0;
  #while (I <= N - 1)
    #local kk[I][0] = (zz[I+1] - zz[I])/(6*hh[I]);
    #local kk[I][1] = zz[I]/2;
    #local kk[I][2] = -hh[I]/6*(zz[I+1] + 2*zz[I]) + (yy[I+1] - yy [I])/hh[I];
    #local I = I + 1;
  #end // while

  #undef hh
  #undef zz

  #local ff = array[Size];

  #local I = 0;
  #while (I <= N - 1)
    #local ff[I] =
      function(t_) {
        yy[I] + (t_ - tt[I])*(kk[I][2] + (t_ - tt[I])*(kk[I][1] + (t_ - tt[I])*kk[I][0]))
      }
    #local I = I + 1;
  #end // while

  function(t_) {
    select(t_ - tt[0], 1, 0, 0)*ff[0](t_) +
    #local I = 0;
    #while (I <= N - 1)
      select(t_ - tt[I], 0, select(t_ - tt[I+1], 1, 0))*ff[I](t_) +
      #local I = I + 1;
    #end // while
    select(t_ - tt[N], 0, 1, 1)*ff[N-1](t_)
  }

#end // macro NaturalCubicSpline_Function1A

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 
// Another way

#declare X_Components = ExtractComponent1A(Points, x);
#declare Y_Components = ExtractComponent1A(Points, y);
#declare Z_Components = ExtractComponent1A(Points, z);

#declare X_Fn = NaturalCubicSpline_Function1A(Times, X_Components);
#declare Y_Fn = NaturalCubicSpline_Function1A(Times, Y_Components);
#declare Z_Fn = NaturalCubicSpline_Function1A(Times, Z_Components);

#declare Intervals = 400;

#declare I = 0;
#while (I < Intervals)
  #declare T = Start + I/Intervals*Span;
  sphere {
    <X_Fn(T), Y_Fn(T), Z_Fn(T)>, 0.04
    pigment { color Yellow }
  }
  #declare I = I + 1;
#end // while

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 

background { color <0.05, 0.10, 0.10> }

light_source {
  5*<-1, 6, -2>
  color White
  shadowless
}

camera {
  location 8*<1, 1, -1>
  look_at <2, 1, 2>
}

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7 ======= 8 ======= 9 ======= 10 

